Module Comunicacion
    'Este modulo implementa las funciones con los algoritmos par inicializar, y comunicarse con 
    'el microcontrolador por tal de intercambiar datos, programarlo, leerlo o verificarlo.
    Public delay As Integer = 1000

    Public Sub SetXtalInitialize()
        EstadoControl = EstadoControl Or BinANat("00000100")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub ClearXtalInitialize()
        EstadoControl = EstadoControl And BinANat("11111011")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub SetResetNotSelect()
        EstadoControl = EstadoControl And BinANat("11110111")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub ClearResetNotSelect()
        EstadoControl = EstadoControl Or BinANat("00001000")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub SetMosiData5()
        EstadoData = EstadoData Or BinANat("00100000")
        Out(DireccionData, EstadoData)
    End Sub

    Public Sub ClearMosiData5()
        EstadoData = EstadoData And BinANat("11011111")
        Out(DireccionData, EstadoData)
    End Sub

    Public Sub SetSckData7()
        EstadoData = EstadoData Or BinANat("10000000")
        Out(DireccionData, EstadoData)
    End Sub

    Public Sub ClearSckData7()
        EstadoData = EstadoData And BinANat("01111111")
        Out(DireccionData, EstadoData)
    End Sub

    Public Function GetMisoAck() As Boolean
        GetMisoAck = BinANat("01000000") And Inp(DireccionStatus)
    End Function

    Public Sub PausaActiva(ByVal iteraciones As Integer)
        Dim j As Integer

        For j = 0 To iteraciones
        Next j

    End Sub


    'Procedimiento que genera numero clocks en XTAL1
    Public Sub GeneraClocks(ByVal numero_de_clocks As Integer, ByVal pausa As Integer)
        Dim i As Integer

        For i = 0 To numero_de_clocks
            ClearXtalInitialize()
            PausaActiva(pausa)
            SetXtalInitialize()
            PausaActiva(pausa)
        Next i

    End Sub


    'Procedimiento que realiza la secuencia de inicializacion del AVR
    Public Sub ActivaProgramacionAVR()
        'Segun el datasheet, primero hay que dejar estas lineas a 0 (Reset,Sck) y 
        'despues(activar) Vcc, no obstante he comprobado que esto ultimo no es necesario,
        'es decir que se puede mantener Vcc activada todo el rato, siempre y cuando 
        'se de el pulso de reset en la activacion tal como se explica mas abajo.

        'Se ponen las lineas Reset y Sck a 0.
        ClearResetNotSelect()
        ClearSckData7()
        'Una vez Reset y Sck estan a 0 se avisa de que se puede activar Vcc
        MsgBox("Ya se puede activar Vcc")
        'Se aplica un pulso de clock a la senyal Xtal1
        GeneraClocks(1, 200 * delay) 'Al ser una etapa critica, en la fase de activacion, la duracion de los pulsos de clock es 200 veces superior a la druaccion en el resto de fases
        'Se da un pulso de reset. Esto solo es necesario cuando no se pueda
        'garantizar que la seal SCK se mantiene a 0 en el momento de activar
        'Vcc. En un principio el diseo de este proyecto si que lo garantiza
        'aunque se da el pulso por si fallara la activacion de Vcc.
        SetResetNotSelect()
        GeneraClocks(16, 200 * delay) 'Al ser una etapa critica, en la fase de activacion, la duracion de los pulsos de clock es 200 veces superior a la druaccion en el resto de fases
        ClearResetNotSelect()
        'Tras generar el pulso hacemos una pausa de 40msg en realidad con 20 hay suficiente        
        Pausa(20)

    End Sub


    'Procedimiento que permite enviar o recibir datos durante la programacion del AVR
    'ValEnviat contiene el byte a enviar en forma de cadena con el valor en binario
    'ValRebut es una cadena con el valor binario contestado por el microcontrolador
    Public Sub Comunica(ByVal ValEnviado As String, ByRef ValRecibido As String)
        Dim i As Integer

        'Se envian los bits de ValEnviado de mas a menos peso
        For i = 7 To 0 Step -1
            ClearSckData7() 'se baja la seal SCK
            GeneraClocks(4, delay)
            'Se pone el bit a enviar en MOSI
            If ByteBitOut(ValEnviado, i) Then
                SetMosiData5() 'se envia un 1
            Else
                ClearMosiData5() 'se envia un 0
            End If
            GeneraClocks(4, delay)
            SetSckData7() 'se levanta la seal SCK y el micro captura el valor
            GeneraClocks(4, delay)
            'Justo antes de bajar SCK se lee lo que llega por MISO
            ByteBitIn(ValRecibido, i, GetMisoAck()) ' si GetMisoAck es True se guarda un 1 si es False un 0
        Next i

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Programing  Enable 
    'Instruction. Retorna por referencia el echo respondido por el microcontrolador.Tambien retorna 
    'un True o un False, en funcion de si la activacion del modo de programacion se ha ejecutado 
    'correctamente o no, es decir en funcion de si el echo del microcontrolador es correcto o no.
    Public Function ComandoProgrammingEnable(ByRef echo As String) As Boolean
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        Comunica("10101100", cad_aux_echo)
        echo = cad_aux_echo
        Comunica("01010011", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("01010101", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("10101010", cad_aux_echo)
        echo = echo + cad_aux_echo
        'Se devuelve un True o un False en funcion de si la secuencia de echo 
        'respeusta por el micrcontrolador es la secuencia de echo correcta o no.
        ComandoProgrammingEnable = (echo = "00000000000000000101001101010101")
        'Pausa(100) 'Se espera un pequeo intervalo para permitir al microcontrolador inicializarse

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Chip
    'Erase Instruction. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoChipErase(ByRef echo As String)
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        Comunica("10101100", cad_aux_echo)
        echo = cad_aux_echo
        Comunica("10000000", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("00000000", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("00000000", cad_aux_echo)
        echo = echo + cad_aux_echo
        'Tras enviar el comando Chip Erase hay que esperar 20 ms aprox. En realidad este 
        'tiempo depende de la tension a la que este alimentado el microcontrolador.
        Pausa(20)

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory
    'Instruction correspondiente a los 8 bits altos. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryAlto(ByVal direccion As String, ByVal dato As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se envia el valor high
        cad_aux_envio = "01001000" + direccion + dato
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory
    'Instruction correspondiente a los 8 bits bajos. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryBajo(ByVal direccion As String, ByVal dato As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se envia el valor low
        cad_aux_envio = "01000000" + direccion + dato
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Program Memory Instruction (byte alto). Retorna por referencia el echo completo
    'respondido por el microcontrolador. El resultado de la funcion es la cadena con
    'el valor leida de la direccion de la program memory indicada.
    Public Function ComandoReadProgramMemoryAlto(ByVal direccion As String, ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor alto
        cad_aux_envio = "00101000" + direccion + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'almacenado en la direccion consultada de la memoria de programa.
        ComandoReadProgramMemoryAlto = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Program Memory Instruction (byte bajo). Retorna por referencia el echo completo
    'respondido por el microcontrolador. El resultado de la funcion es la cadena
    'con el valor leida de la direccion de la program memory indicada.
    Public Function ComandoReadProgramMemoryBajo(ByVal direccion As String, ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor bajo
        cad_aux_envio = "00100000" + direccion + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'almacenado en la direccion consultada de la memoria de programa.
        ComandoReadProgramMemoryBajo = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Eprom Memory
    'Instruction. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteEpromMemory(ByVal direccion As String, ByVal dato As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se envia el valor
        cad_aux_envio = "11000000" + direccion + dato
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Eprom Memory Instruction. Retorna por referencia el echo completo respondido 
    'por el microcontrolador. El resultado de la funcion es la cadena con el valor
    'leido de la direccion de la program memory indicada.
    Public Function ComandoReadEpromMemory(ByVal direccion As String, ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor
        cad_aux_envio = "10100000" + direccion + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'almacenado en la direccion consultada de la memoria de programa.
        ComandoReadEpromMemory = Mid(echo, Len(echo) - 7, 8)

    End Function


End Module
