Module BajoNivel

    ' Constantes para el Atmel AVR 8Ks (8515 8515L 8535 8535L ...)
    Public Const DELAY_AVR8K As Integer = 1000
    Public Const CLOCKS_PULSO_SCK_AVR8K As Integer = 8

    Public Const TWD_FLASH_AVR8K As Integer = 5
    Public Const TWD_EPROM_AVR8K As Integer = 10
    Public Const TRD_EPROM_AVR8K As Integer = 10
    Public Const TWD_ERASE_AVR8K As Integer = 20
    Public Const TW_INIT_AVR8K As Integer = 20

    ' Constantes para el Atmel ATmega 8Ks (8515 8515L 8535 8535L ...)
    Public Const DELAY_ATMEGA8K As Integer = 1
    Public Const CLOCKS_PULSO_SCK_ATMEGA8K As Integer = 8

    Public Const TWD_FLASH_ATMEGA8K As Integer = 1
    Public Const TWD_EPROM_ATMEGA8K As Integer = 2
    Public Const TRD_EPROM_ATMEGA8K As Integer = 1
    Public Const TWD_ERASE_ATMEGA8K As Integer = 4
    Public Const TW_INIT_ATMEGA8K As Integer = 8

    ' Constantes para el Atmel ATmega 16Ks
    Public Const DELAY_ATMEGA16K As Integer = 1
    Public Const CLOCKS_PULSO_SCK_ATMEGA16K As Integer = 8

    Public Const TWD_FLASH_ATMEGA16K As Integer = 1
    Public Const TWD_EPROM_ATMEGA16K As Integer = 2
    Public Const TRD_EPROM_ATMEGA16K As Integer = 1
    Public Const TWD_ERASE_ATMEGA16K As Integer = 4
    Public Const TW_INIT_ATMEGA16K As Integer = 8

    ' Constantes para el Atmel ATmega 32Ks
    Public Const DELAY_ATMEGA32K As Integer = 1
    Public Const CLOCKS_PULSO_SCK_ATMEGA32K As Integer = 8

    Public Const TWD_FLASH_ATMEGA32K As Integer = 1
    Public Const TWD_EPROM_ATMEGA32K As Integer = 2
    Public Const TRD_EPROM_ATMEGA32K As Integer = 1
    Public Const TWD_ERASE_ATMEGA32K As Integer = 4
    Public Const TW_INIT_ATMEGA32K As Integer = 8

    ' Constantes para el Atmel ATmega 64Ks (64 64L ...)
    Public Const DELAY_ATMEGA64K As Integer = 1
    Public Const CLOCKS_PULSO_SCK_ATMEGA64K As Integer = 8

    Public Const TWD_FLASH_ATMEGA64K As Integer = 1
    Public Const TWD_EPROM_ATMEGA64K As Integer = 2
    Public Const TRD_EPROM_ATMEGA64K As Integer = 1
    Public Const TWD_ERASE_ATMEGA64K As Integer = 4
    Public Const TW_INIT_ATMEGA64K As Integer = 8

    Public Twd_Flash As Integer
    Public Twd_Eprom As Integer
    Public Trd_Eprom As Integer
    Public Twd_Erase As Integer
    Public Tw_Init As Integer

    'Librerias de acceso al puerto paralelo
    Public Declare Function Inp Lib "inpout32.dll" Alias "Inp32" (ByVal PortAddress As Integer) As Integer
    Public Declare Sub Out Lib "inpout32.dll" Alias "Out32" (ByVal PortAddress As Integer, ByVal Value As Integer)
    'Direcciones de los registros del puerto paralelo
    Public Const DireccionData As Integer = 888
    Public Const DireccionStatus As Integer = 889
    Public Const DireccionControl As Integer = 890
    'Variables que contienen el estado de las lineas de los diferentes registros
    Public EstadoData As Byte
    Public EstadoStatus As Byte
    Public EstadoControl As Byte
    'Librerias para el control de las temporizaciones
    Public Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
    Public Declare Function GetTickCount Lib "kernel32" () As Long


    'Funcion que realiza una Pausa de TmEspera milisegundos
    Public Sub Pausa(ByVal TmEspera As Long)
        Dim TmFin As Long

        'GetTickCount function retrieves the number of milliseconds that have elapsed since the system was started. 
        TmFin = GetTickCount() + TmEspera

        Do While GetTickCount() < TmFin
            'DoEvents() 'Aadir esto si se quiere que el sistema pueda ejecutar otros procesos
        Loop

    End Sub


    'Funcion que realiza una Pausa de TmEspera milisegundos
    'Esta funcion realiza la pausa del proceso actual pero permite al
    'programa seguir atendiendo los eventos
    Public Sub PausaProceso(ByVal TmEspera As Long)
        Dim TmFin As Long

        TmFin = GetTickCount + TmEspera
        Do While GetTickCount < TmFin
            'DoEvents() 'Aadir esto si se quiere que el sistema pueda ejecutar otros procesos
        Loop

    End Sub


    'Funcion que recibe una representacion binaria y retorna su valor
    Public Function BinANat(ByVal ValBin As String) As Integer

        Dim ValAn As Integer
        Dim PosBin As Integer
        ValAn = 0
        PosBin = 1
        Do
            If PosBin = (Len(ValBin) + 1) Then Exit Do
            If Right(Left(ValBin, PosBin), 1) = "1" Then
                ValAn = (ValAn + 0.5) * 2
            Else
                ValAn = ValAn * 2
            End If
            PosBin = PosBin + 1
        Loop
        BinANat = ValAn

    End Function


    'Funcion que recibe un natural y retorna su representacion binaria.
    Public Function NatABin(ByVal ValNat As Integer) As String
        Dim ValNat2 As String
        Dim ValNat1 As Integer
        Dim Res As Integer
        Dim Bin As String

        ValNat2 = ""
        ValNat1 = ValNat
        Do
            If ValNat1 = 0 Then
                Exit Do
            End If
            Res = ValNat1 Mod 2
            Bin = Res
            ValNat2 = Bin + ValNat2
            ValNat1 = Int(ValNat1 / 2)
        Loop
        NatABin = ValNat2

    End Function


    'Funcion que recibe la representacion binaria de un byte (8 bits)
    'y  retorna su representacion hexadecimal. Si la representacion binaria
    'recibida tiene menos de 8 digitos binarios se aaden '0's hasta que 
    'tenga 8, si tiene 8 o mas digitos se cogen los 8 de menos peso. Recordar 
    'que cuatro digitos binarios son un digito hexadecimal, por tanto 8 digitos
    'binarios son 2 digitos hexadecimales.
    Public Function ByteBinAHex(ByVal ByteBin As String) As String

        'Se comprueba la coherencia de la variable ByteBin recibida
        If Len(ByteBin) < 8 Then
            'Si tiene menos de 8 digitos se rellena con '0's
            While Len(ByteBin) < 8
                ByteBin = "0" + ByteBin
            End While
        Else
            'Si tiene 8 o mas digitos se toman solo los 8 digitos de menos peso
            ByteBin = Mid(ByteBin, Len(ByteBin) - 7, 8)
        End If
        ByteBinAHex = NibleBinAHex(Mid(ByteBin, 1, 4))
        ByteBinAHex = ByteBinAHex + NibleBinAHex(Mid(ByteBin, 5, 4))

    End Function


    'Funcion que recibe la representacion binaria de una word (16 bits)
    'y  retorna su representacion hexadecimal. Si la representacion binaria
    'recibida tiene menos de 16 digitos binarios se aaden '0's hasta que 
    'tenga 16, si tiene 16 o mas digitos se cogen los 16 de menos peso. Recordar 
    'que cuatro digitos binarios son un digito hexadecimal, por tanto 16 digitos
    'binarios son 4 digitos hexadecimales.
    Public Function WordBinAHex(ByVal WordBin As String) As String

        'Se comprueba la coherencia de la variable WordBin recibida
        If Len(WordBin) < 16 Then
            'Si tiene menos de 16 digitos se rellena con '0's
            While Len(WordBin) < 16
                WordBin = "0" + WordBin
            End While
        Else
            'Si tiene 16 o mas digitos se toman solo los 16 digitos de menos peso
            WordBin = Mid(WordBin, Len(WordBin) - 15, 16)
        End If
        WordBinAHex = NibleBinAHex(Mid(WordBin, 1, 4))
        WordBinAHex = WordBinAHex + NibleBinAHex(Mid(WordBin, 5, 4))
        WordBinAHex = WordBinAHex + NibleBinAHex(Mid(WordBin, 9, 4))
        WordBinAHex = WordBinAHex + NibleBinAHex(Mid(WordBin, 13, 4))

    End Function



    'Funcion que recibe la representacion binaria de un nible (4 bits)
    'y  retorna su representacion hexadecimal. Si la representacion binaria
    'recibida tiene menos de 4 digitos binarios se aaden '0's hasta que 
    'tenga 4, si tiene 4 o mas digitos se cogen los 4 de menos peso. Recordar 
    'que cuatro digitos binarios son un digito hexadecimal.
    Public Function NibleBinAHex(ByVal NibleBin As String) As String

        'Se comprueba la coherencia de la variable NibleBin recibida
        If Len(NibleBin) < 4 Then
            'Si tiene menos de 4 digitos se rellena con '0's
            While Len(NibleBin) < 4
                NibleBin = "0" + NibleBin
            End While
        Else
            'Si tiene 4 o mas digitos se toman solo los 4 digitos de menos peso
            NibleBin = Mid(NibleBin, Len(NibleBin) - 3, 4)
        End If

        'Se realiza la conversion
        Select Case NibleBin
            Case "0000"
                NibleBinAHex = "0"
            Case "0001"
                NibleBinAHex = "1"
            Case "0010"
                NibleBinAHex = "2"
            Case "0011"
                NibleBinAHex = "3"
            Case "0100"
                NibleBinAHex = "4"
            Case "0101"
                NibleBinAHex = "5"
            Case "0110"
                NibleBinAHex = "6"
            Case "0111"
                NibleBinAHex = "7"
            Case "1000"
                NibleBinAHex = "8"
            Case "1001"
                NibleBinAHex = "9"
            Case "1010"
                NibleBinAHex = "A"
            Case "1011"
                NibleBinAHex = "B"
            Case "1100"
                NibleBinAHex = "C"
            Case "1101"
                NibleBinAHex = "D"
            Case "1110"
                NibleBinAHex = "E"
            Case "1111"
                NibleBinAHex = "F"
            Case Else
                NibleBinAHex = "F"
        End Select

    End Function



    'Funcion que recibe la representacion hexadecimal de un byte y retorna su representacon binaria.
    Public Function ByteHexABin(ByVal ByteHex As String) As String

        ByteHexABin = NibleHexABin(Mid(ByteHex, 1, 1)) + NibleHexABin(Mid(ByteHex, 2, 1))

    End Function


    'Funcion que recibe un natural y retorna la representacion binaria de este en formato byte
    Public Function NatAByteBin(ByVal Nat As String) As String

        NatAByteBin = NatABin(Nat)

        'Se verifica que la cadena retornada este formada por 8 caracteres
        If Len(NatAByteBin) < 8 Then
            'Si la longitud de cad_aux es menor de 16 se rellenan los caracters que faltan con "0"
            While Len(NatAByteBin) < 8
                NatAByteBin = "0" + NatAByteBin
            End While
        Else
            'Si la longitud de la cadena es mayor de 8 se cogen solo los 8 caracteres correspondientes a los 8 bits mas bajos
            NatAByteBin = Mid(NatAByteBin, Len(NatAByteBin) - 7, 8)
        End If

    End Function


    'Funcion que recibe un natural y retorna la representacion binaria de este en formato word
    Public Function NatAWordBin(ByVal Nat As String) As String

        NatAWordBin = NatABin(Nat)

        'Se verifica que la cadena retornada este formada por 16 caracteres
        If Len(NatAWordBin) < 16 Then
            'Si la longitud de cad_aux es menor de 16 se rellenan los caracters que faltan con "0"
            While Len(NatAWordBin) < 16
                NatAWordBin = "0" + NatAWordBin
            End While
        Else
            'Si la longitud de la cadena es mayor de 16 se cogen solo los 16 caracteres correspondientes a los 16 bits mas bajos
            NatAWordBin = Mid(NatAWordBin, Len(NatAWordBin) - 15, 16)
        End If

    End Function



    'Funcion que recibe la representacion hexadecimal de un nible (4 bits)
    'y retorna su representacion binaria. Si la representacion hexadecimal
    'recibida esta vacia se les asigna  '0', si tiene 1 digito o mas se coge el
    'de menos peso. Recordar que cuatro digitos binarios son un digito hexadecimal.
    Public Function NibleHexABin(ByVal NibleHex As String) As String

        'Se comprueba la coherencia de la variable NibleHex recibida
        If NibleHex = "" Then
            'Si la cadena tiene menos de un digito hexadecimal, es decir ninguno,  se le asigna un 0
            NibleHex = ""
        Else
            'Si tiene 1 o mas digitos se toma solo el de menos peso
            NibleHex = Mid(NibleHex, Len(NibleHex), 1)
        End If

        'Se realiza la conversion
        Select Case NibleHex
            Case "0"
                NibleHexABin = "0000"
            Case "1"
                NibleHexABin = "0001"
            Case "2"
                NibleHexABin = "0010"
            Case "3"
                NibleHexABin = "0011"
            Case "4"
                NibleHexABin = "0100"
            Case "5"
                NibleHexABin = "0101"
            Case "6"
                NibleHexABin = "0110"
            Case "7"
                NibleHexABin = "0111"
            Case "8"
                NibleHexABin = "1000"
            Case "9"
                NibleHexABin = "1001"
            Case "A"
                NibleHexABin = "1010"
            Case "B"
                NibleHexABin = "1011"
            Case "C"
                NibleHexABin = "1100"
            Case "D"
                NibleHexABin = "1101"
            Case "E"
                NibleHexABin = "1110"
            Case "F"
                NibleHexABin = "1111"
            Case Else
                NibleHexABin = "1111"
        End Select

    End Function



    'Funcion que recibe la expresion hexadecimal de un nible (4 bits) y
    'retorna su valor. Si la representacion hexadecimal recibida esta 
    'vacia se les asigna  '0', si tiene 1 digito o mas se coge el de 
    'menos peso. Recordar que cuatro digitos binarios son un digito hexadecimal.
    Public Function NibleHexANat(ByVal NibleHex As String) As Integer

        'Se comprueba la coherencia de la variable NibleHex recibida
        If NibleHex = "" Then
            'Si la cadena tiene menos de un digito hexadecimal, es decir ninguno, se le asigna un 0
            NibleHex = ""
        Else
            'Si tiene 1 o mas digitos se toma solo el de menos peso
            NibleHex = Mid(NibleHex, Len(NibleHex), 1)
        End If

        'Se realiza la conversion
        Select Case NibleHex
            Case "0"
                NibleHexANat = 0
            Case "1"
                NibleHexANat = 1
            Case "2"
                NibleHexANat = 2
            Case "3"
                NibleHexANat = 3
            Case "4"
                NibleHexANat = 4
            Case "5"
                NibleHexANat = 5
            Case "6"
                NibleHexANat = 6
            Case "7"
                NibleHexANat = 7
            Case "8"
                NibleHexANat = 8
            Case "9"
                NibleHexANat = 9
            Case "A"
                NibleHexANat = 10
            Case "B"
                NibleHexANat = 11
            Case "C"
                NibleHexANat = 12
            Case "D"
                NibleHexANat = 13
            Case "E"
                NibleHexANat = 14
            Case "F"
                NibleHexANat = 15
            Case Else
                NibleHexANat = 0
        End Select

    End Function



    'Funcion que recibe la representacion hexadecimal de un byte y retorna su valor.
    'Si la representacion hexadecimal recibida tiene menos de 2 digitos hexadecimales
    'aaden '0's hasta que tenga 2, si tiene 2 o mas se cogen los 2 de menos peso. 
    'Recordar que 2 digitos hexadecimal son 8 digtos binarios es decir 1 byte.
    Public Function ByteHexANat(ByVal ByteHex As String) As Integer

        'Se comprueba la coherencia de la variable NibleBin recibida
        If Len(ByteHex) < 2 Then
            'Si tiene menos de 2 digitos se rellena con '0's
            While Len(ByteHex) < 2
                ByteHex = "0" + ByteHex
            End While
        Else
            'Si tiene 2 o mas digitos se toman solo los 4 digitos de menos peso
            ByteHex = Mid(ByteHex, Len(ByteHex) - 1, 2)
        End If

        'Se realiza la conversion
        ByteHexANat = NibleHexANat(Mid(ByteHex, 2, 1))
        ByteHexANat = ByteHexANat + 16 * NibleHexANat(Mid(ByteHex, 1, 1))

    End Function



    'Funcion que recibe la representacion hexadecimal de una word y retorna su valor.
    'Si la representacion hexadecimal recibida tiene menos de 4 digitos hexadecimales
    'aaden '0's hasta que tenga 4, si tiene 4 o mas se cogen los 4 de menos peso. 
    'Recordar que 4 digitos hexadecimal son 16 digtos binarios es decir 2 bytes.
    Public Function WordHexANat(ByVal WordHex As String) As Integer

        'Se comprueba la coherencia de la variable NibleBin recibida
        If Len(WordHex) < 4 Then
            'Si tiene menos de 2 digitos se rellena con '0's
            While Len(WordHex) < 4
                WordHex = "0" + WordHex
            End While
        Else
            'Si tiene 2 o mas digitos se toman solo los 4 digitos de menos peso
            WordHex = Mid(WordHex, Len(WordHex) - 3, 4)
        End If

        'Se realiza la conversion
        WordHexANat = NibleHexANat(Mid(WordHex, 4, 1))
        WordHexANat = WordHexANat + 16 * NibleHexANat(Mid(WordHex, 3, 1))
        WordHexANat = WordHexANat + 256 * NibleHexANat(Mid(WordHex, 2, 1))
        WordHexANat = WordHexANat + 4096 * NibleHexANat(Mid(WordHex, 1, 1))

    End Function




    'Funcion que recibe un natural y retorna la representacion hexadecimal de
    'de este en formato de word. Si el valor natural supera el rango representable
    'por una word se devuelven solo los bits de menos peso que caben dentro 
    'de la word, el resto se pierden.
    Public Function NatAWordHex(ByVal Nat As Integer) As String
        'Se convierte el valor en una cadena con la representacion hexadecimal
        NatAWordHex = Hex(Nat)

        'Se verifica que la cadena retornada este formada por 4 caracteres
        If Len(NatAWordHex) < 4 Then
            'Si la longitud de cad_aux es menor de 4 se rellenan los caracters que faltan con "0"
            While Len(NatAWordHex) < 4
                NatAWordHex = "0" + NatAWordHex
            End While
        Else
            'Si la longitud de cad_aux es mayor de 4 se cogen solo los 4 caracteres correspondientes a los 16 bits mas bajos
            NatAWordHex = Mid(NatAWordHex, Len(NatAWordHex) - 3, 4)
        End If

    End Function




    'Funcion que recibe un natural y retorna la representacion hexadecimal de
    'de este en formato de byte. Si el valor natural supera el rango representable
    'por un byte se devuelven solo los bits de menos peso que caben dentro 
    'del byte, el resto se pierde.
    Public Function NatAByteHex(ByVal Nat As Integer) As String
        'Se convierte el valor en una cadena con la representacion hexadecimal
        NatAByteHex = Hex(Nat)

        'Se verifica que la cadena retornada este formada por 4 caracteres
        If Len(NatAByteHex) < 2 Then
            'Si la longitud de cad_aux es menor de 4 se rellenan los caracters que faltan con "0"
            While Len(NatAByteHex) < 2
                NatAByteHex = "0" + NatAByteHex
            End While
        Else
            'Si la longitud de cad_aux es mayor de 2 se cogen solo los 2 caracteres correspondientes a los 8 bits mas bajos
            NatAByteHex = Mid(NatAByteHex, Len(NatAByteHex) - 1, 2)
        End If

    End Function




    'Procedimiento que situa el Bit recibido en la posicion Pos de la cadena ByteValBin
    'Pos: 7 a 0
    'Bit: Verdadero=1 Falso=0
    Public Sub ByteBitIn(ByRef ByteValBin As String, ByVal Pos As Integer, ByVal Bit As Boolean)
        '      "01010101"
        'Bit    76543210
        'Pos    76543210
        'Cadena 12345678
        If Bit Then
            Mid(ByteValBin, 8 - Pos, 1) = "1"
        Else
            Mid(ByteValBin, 8 - Pos, 1) = "0"
        End If

    End Sub



    'Funcion que retorna el Bit que se encuentra en la posicion Pos de la cadena ByteValBin
    'Pos: 7 a 0
    'Bit: Verdadero=1 Falso=0
    Public Function ByteBitOut(ByVal ByteValBin As String, ByVal Pos As Integer) As Boolean
        Dim retorno As Boolean
        '      "01010101"
        'Bit    76543210
        'Pos    76543210
        'Cadena 12345678
        If Mid(ByteValBin, 8 - Pos, 1) = "1" Then
            retorno = True
        Else
            retorno = False
        End If
        ByteBitOut = retorno

    End Function







End Module
