Module Comunicacion


    'Este modulo implementa las funciones con los algoritmos par inicializar, y comunicarse con 
    'el microcontrolador por tal de intercambiar datos, programarlo, leerlo o verificarlo.
    Public delay As Integer = 1000
    Public clocks_pulso_sck As Integer = 8  ' es el numero de ciclos de cristal del micro durante los que se va a mantener levantada las seal SCK

    Public Sub SetXtalInitialize()
        EstadoControl = EstadoControl Or BinANat("00000100")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub ClearXtalInitialize()
        EstadoControl = EstadoControl And BinANat("11111011")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub SetResetNotSelect()
        EstadoControl = EstadoControl And BinANat("11110111")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub ClearResetNotSelect()
        EstadoControl = EstadoControl Or BinANat("00001000")
        Out(DireccionControl, EstadoControl)
    End Sub

    Public Sub SetMosiData5()
        EstadoData = EstadoData Or BinANat("00100000")
        Out(DireccionData, EstadoData)
    End Sub

    Public Sub ClearMosiData5()
        EstadoData = EstadoData And BinANat("11011111")
        Out(DireccionData, EstadoData)
    End Sub

    Public Sub SetSckData7()
        EstadoData = EstadoData Or BinANat("10000000")
        Out(DireccionData, EstadoData)
    End Sub

    Public Sub ClearSckData7()
        EstadoData = EstadoData And BinANat("01111111")
        Out(DireccionData, EstadoData)
    End Sub

    Public Function GetMisoAck() As Boolean
        GetMisoAck = BinANat("01000000") And Inp(DireccionStatus)
    End Function

    Public Sub PausaActiva(ByVal iteraciones As Integer)
        Dim j As Integer
        Dim k As Byte

        k = 0
        For j = 0 To iteraciones
            k = k + 2
            k = k - 2
        Next j

    End Sub


    'Procedimiento que genera numero clocks en XTAL1 para "mover" el micro
    Public Sub GeneraClocks(ByVal numero_de_clocks As Integer, ByVal pausa As Integer)
        Dim i As Integer

        For i = 0 To numero_de_clocks
            ClearXtalInitialize()
            PausaActiva(pausa)
            SetXtalInitialize()
            PausaActiva(pausa)
        Next i

    End Sub


    'Procedimiento que realiza la secuencia de inicializacion del AVR
    Public Sub ActivaProgramacionAVR()
        'Segun el datasheet, primero hay que dejar estas lineas a 0 (Reset,Sck) y 
        'despues(activar) Vcc, no obstante he comprobado que esto ultimo no es necesario,
        'es decir que se puede mantener Vcc activada todo el rato, siempre y cuando 
        'se de el pulso de reset en la activacion tal como se explica mas abajo.

        'Se ponen las lineas Reset y Sck a 0.
        ClearResetNotSelect()
        ClearSckData7()
        'Una vez Reset y Sck estan a 0 se avisa de que se puede activar Vcc
        MsgBox("Ya se puede activar Vcc")
        'Se aplica un pulso de clock a la senyal Xtal1
        GeneraClocks(1, 200 * delay) 'Al ser una etapa critica, en la fase de activacion, la duracion de los pulsos de clock es 200 veces superior a la druaccion en el resto de fases
        'Se da un pulso de reset. Esto solo es necesario cuando no se pueda
        'garantizar que la seal SCK se mantiene a 0 en el momento de activar
        'Vcc. En un principio el diseo de este proyecto si que lo garantiza
        'aunque se da el pulso por si fallara la activacion de Vcc.
        SetResetNotSelect()
        GeneraClocks(16, 200 * delay) 'Al ser una etapa critica, en la fase de activacion, la duracion de los pulsos de clock es 200 veces superior a la druaccion en el resto de fases
        ClearResetNotSelect()
        'Tras generar el pulso hacemos una pausa de 40msg en realidad con 20 hay suficiente        
        Pausa(2 * Tw_Init)

    End Sub


    'Procedimiento que permite enviar o recibir datos durante la programacion del AVR
    'ValEnviat contiene el byte a enviar en forma de cadena con el valor en binario
    'ValRebut es una cadena con el valor binario contestado por el microcontrolador
    Public Sub Comunica(ByVal ValEnviado As String, ByRef ValRecibido As String)
        Dim i As Integer

        'Se envian los bits de ValEnviado de mas a menos peso
        For i = 7 To 0 Step -1
            ClearSckData7() 'se baja la seal SCK
            GeneraClocks(clocks_pulso_sck, delay)
            'Se pone el bit a enviar en MOSI
            If ByteBitOut(ValEnviado, i) Then
                SetMosiData5() 'se envia un 1
            Else
                ClearMosiData5() 'se envia un 0
            End If
            GeneraClocks(clocks_pulso_sck, delay)
            SetSckData7() 'se levanta la seal SCK y el micro captura el valor
            GeneraClocks(clocks_pulso_sck, delay)
            'Justo antes de bajar SCK se lee lo que llega por MISO
            ByteBitIn(ValRecibido, i, GetMisoAck()) ' si GetMisoAck es True se guarda un 1 si es False un 0
        Next i

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Programing  Enable 
    'Instruction. Retorna por referencia el echo respondido por el microcontrolador.Tambien retorna 
    'un True o un False, en funcion de si la activacion del modo de programacion se ha ejecutado 
    'correctamente o no, es decir en funcion de si el echo del microcontrolador es correcto o no.
    Public Function ComandoProgrammingEnable(ByRef echo As String) As Boolean
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        Comunica("10101100", cad_aux_echo)
        echo = cad_aux_echo
        Comunica("01010011", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("01010101", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("10101010", cad_aux_echo)
        echo = echo + cad_aux_echo

        'Se devuelve un True o un False en funcion de si la secuencia de echo 
        'respeusta por el micrcontrolador es la secuencia de echo correcta o no.
        ComandoProgrammingEnable = (echo = "00000000000000000101001101010101")

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Chip
    'Erase Instruction. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoChipErase(ByRef echo As String)
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        Comunica("10101100", cad_aux_echo)
        echo = cad_aux_echo
        Comunica("10000000", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("00000000", cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica("00000000", cad_aux_echo)
        echo = echo + cad_aux_echo
        'Tras enviar el comando Chip Erase hay que esperar 20 ms aprox. En realidad este 
        'tiempo depende de la tension a la que este alimentado el microcontrolador.
        Pausa(2 * Twd_Erase)

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory
    'Instruction correspondiente a los 8 bits altos. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryAltoAVR(ByVal direccion As String, ByVal dato As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se envia el valor high
        cad_aux_envio = "01001000" + direccion + dato
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory
    'Instruction correspondiente a los 8 bits bajos. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryBajoAVR(ByVal direccion As String, ByVal dato As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se envia el valor low
        cad_aux_envio = "01000000" + direccion + dato
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Load Program Memory Page de los 8
    'bits bajos y 8 bits altos de una instruccion. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoLoadProgramMemoryPage(ByVal direccion As String, ByVal datoL As String, ByVal datoH As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        '0100H000 aaaa aaaa bbbb bbbb oooo oooo 

        echo = ""
        'se envia el valor Low
        cad_aux_envio = "01000000" + direccion + datoL
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

        Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet

        'se envia el valor High
        cad_aux_envio = "01001000" + direccion + datoH
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

        Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory Page en los
    'micros ATMega de 8ks por tal de escribir en Flash la pagina cargada. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryPageATMega8K(ByVal direccion As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        'AT-Mega8K Write Program Memory Page Instruction
        '01001100 0000 aaaa bbbx xxxx xxxxxxxx

        echo = ""
        'se envia la instruccion
        'ATENCION: como direccion la recibimos con 8 bits no sobrar 1 bit de direccion por arriba ya que en este modelo 
        'solo se usan 7 bits, asi que quitamos un 0 que al concatenar se sustituira por el de direccion que sobra.
        cad_aux_envio = "01001100" + "000" + direccion + "00000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

        Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory Page en los
    'micros ATMega de 16ks por tal de escribir en Flash la pagina cargada. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryPageATMega16K(ByVal direccion As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        'AT-Mega16 Write Program Memory Page Instruction
        '01001100 000a aaaa aaxx xxxx xxxxxxxx

        echo = ""
        'se envia la instruccion
        'ATENCION: como direccion la recibimos con 8 bits no sobrar 1 bit de direccion por arriba ya que en este modelo 
        'solo se usan 7 bits, asi que quitamos un 0 que al concatenar se sustituira por el de direccion que sobra
        cad_aux_envio = "01001100" + "00" + direccion + "000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

        Pausa(Twd_Flash)

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory Page en los
    'micros ATMega de 32ks por tal de escribir en Flash la pagina cargada. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryPageATMega32K(ByVal direccion As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        'AT-Mega32 Write Program Memory Page Instruction
        '01001100 00aa aaaa bbxx xxxx xxxxxxxx

        echo = ""
        'se envia la instruccion
        'ATENCION: como direccion que recibimos es de 8 bits y la direccion segun la instruccion es de
        '8 bits no hace falta quitar ni realizar nigun apao con los bits de la cadena a enviar y la direccion
        cad_aux_envio = "01001100" + "00" + direccion + "000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

        Pausa(Twd_Flash)

    End Sub



    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Program Memory Page en los
    'micros ATMega de 64ks por tal de escribir en Flash la pagina cargada. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteProgramMemoryPageATMega64K(ByVal direccion As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        'AT-Mega64 Write Program Memory Page Instruction
        '01001100 xaaa aaaa bxxx xxxx xxxxxxxx

        echo = ""
        'se envia la instruccion
        'ATENCION: como direccion que recibimos es de 8 bits y la direccion segun la instruccion es de
        '8 bits no hace falta quitar ni realizar nigun apao con los bits de la cadena a enviar y la direccion
        cad_aux_envio = "01001100" + "0" + direccion + "0000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

        Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Program Memory Instruction (byte alto). Retorna por referencia el echo completo
    'respondido por el microcontrolador. El resultado de la funcion es la cadena con
    'el valor leida de la direccion de la program memory indicada.
    Public Function ComandoReadProgramMemoryAlto(ByVal direccion As String, ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor alto
        cad_aux_envio = "00101000" + direccion + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'almacenado en la direccion consultada de la memoria de programa.
        ComandoReadProgramMemoryAlto = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Program Memory Instruction (byte bajo). Retorna por referencia el echo completo
    'respondido por el microcontrolador. El resultado de la funcion es la cadena
    'con el valor leida de la direccion de la program memory indicada.
    Public Function ComandoReadProgramMemoryBajo(ByVal direccion As String, ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor bajo
        cad_aux_envio = "00100000" + direccion + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'almacenado en la direccion consultada de la memoria de programa.
        ComandoReadProgramMemoryBajo = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write Eprom Memory
    'Instruction. Retorna el echo respondido por el microcontrolador.
    Public Sub ComandoWriteEpromMemory(ByVal direccion As String, ByVal dato As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se envia el valor
        cad_aux_envio = "11000000" + direccion + dato
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Eprom Memory Instruction. Retorna por referencia el echo completo respondido 
    'por el microcontrolador. El resultado de la funcion es la cadena con el valor
    'leido de la direccion de la program memory indicada.
    Public Function ComandoReadEpromMemory(ByVal direccion As String, ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor
        cad_aux_envio = "10100000" + direccion + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'almacenado en la direccion consultada de la memoria de programa.
        ComandoReadEpromMemory = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Fuse Bits. Retorna por referencia el echo completo respondido por el 
    'microcontrolador. Retorna por valor los fuse bits. El comando es el siguiente:
    '   Read Fuse Bits		0101 0000 0000 0000 xxxx xxxx oooo oooo
    Public Function ComandoReadFuseBits(ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor
        cad_aux_envio = "01010000" + "00000000" + "00000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'de los fusebits
        ComandoReadFuseBits = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Fuse High Bits. Retorna por referencia el echo completo respondido por el 
    'microcontrolador. Retorna por valor los fuse high bits. El comando es el siguiente:
    '   Read Fuse High Bits      0101 1000 0000 1000 xxxx xxxx oooo oooo
    Public Function ComandoReadFuseHighBits(ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor
        cad_aux_envio = "01011000" + "00001000" + "00000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'de los fuse high bits
        ComandoReadFuseHighBits = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Read
    'Extended Fuse Bits. Retorna por referencia el echo completo respondido por el 
    'microcontrolador. Retorna por valor los extended fusebits. El comando es el
    'siguiente:
    '   Read Extendend Fuse Bits      0101 0000 0000 1000 xxxx xxxx oooo oooo
    Public Function ComandoReadExtendedFuseBits(ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor
        cad_aux_envio = "01010000" + "00001000" + "00000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'de los fuse high bits
        ComandoReadExtendedFuseBits = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write
    'Fuse Bits. Retorna por referencia el echo completo respondido por el 
    'microcontrolador. El comando es el siguiente:
    '   Write Fuse Bits 	 1010 1100 1010 0000 xxxx xxxx iiii iiii
    Public Sub ComandoWriteFuseBits(ByVal fusebits As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se escribe el valor
        cad_aux_envio = "10101100" + "10100000" + "00000000" + fusebits
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write
    'High Fuse Bits. Retorna por referencia el echo completo respondido por el 
    'microcontrolador. El comando es el siguiente:
    '   Write Fuse High Bits 	 1010 1100 1010 1000 xxxx xxxx iiii iiii
    Public Sub ComandoWriteFuseHighBits(ByVal fusehighbits As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se escribe el valor
        cad_aux_envio = "10101100" + "10101000" + "00000000" + fusehighbits
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write
    'Extended Fuse Bits. Retorna por referencia el echo completo respondido por el 
    'microcontrolador. El comando es el siguiente:
    '   Write Extended Fuse Bits 1010 1100 1010 0100 xxxx xxxx xxxx xxii
    Public Sub ComandoWriteExtendedFuseBits(ByVal extendedfusebits As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se escribe el valor
        cad_aux_envio = "10101100" + "10100100" + "00000000" + extendedfusebits
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que envia al microcontrolador la secuencia correspondiente a la instr Read
    'Lock Bits. Retorna por referencia el echo completo respondido por el microcontrolador.
    'Retorna por valor los lock bits. El comando es el siguiente:
    '   Read Lock Bits		0101 1000 0000 0000 xxxx xxxx xxoo oooo
    Public Function ComandoReadLockBits(ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor
        cad_aux_envio = "01011000" + "00000000" + "00000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'de los lockbits
        ComandoReadLockBits = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que envia al microcontrolador la secuencia correspondiente a la Write
    'Lock Bits. Retorna por referencia el echo completo respondido por el microcontrolador.
    'El comando es el siguiente:
    '   Write Lock Bits		1010 1100 111x xxxx xxxx xxxx 11ii iiii
    Public Sub ComandoWriteLockBits(ByVal lockbits As String, ByRef echo As String)
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        echo = ""
        'se escribe el valor
        cad_aux_envio = "10101100" + "11100000" + "00000000" + lockbits
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo

    End Sub


    'Funcion que al microcontrolador la secuencia correspondiente a la Read
    'CalibrationByte. Retorna por referencia el echo completo respondido por el 
    'microcontrolador. Retorna por valor el ReadCalibrationByte. El comando es el siguiente:
    '   Read Calibration Byte   0011 1000 00xx xxxx 0000 0000 oooo oooo
    Public Function ComandoReadCalibrationByte(ByRef echo As String) As String
        Dim cad_aux_envio As String
        Dim cad_aux_echo As String = "00000000"

        echo = ""
        'se consulta el valor
        cad_aux_envio = "00111000" + "00000000" + "00000000" + "00000000"
        Comunica(Mid(cad_aux_envio, 1, 8), cad_aux_echo)
        echo = cad_aux_echo
        Comunica(Mid(cad_aux_envio, 9, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 17, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        Comunica(Mid(cad_aux_envio, 25, 8), cad_aux_echo)
        echo = echo + cad_aux_echo
        'Los ultimos 8 bits del echo del micrcontrolador contienen el valor
        'del calibration byte
        ComandoReadCalibrationByte = Mid(echo, Len(echo) - 7, 8)

    End Function


    'Funcion que retorna un TRUE si el fuse de peso i se encuentra activo, y retorna
    'un FALSE si se encuentra inactivo. Recordar que para los fuse bits:
    '           '0' = programmed,   TRUE
    '           '1' = unprogrammed. FALSE
    Public Function ObtenEstadoDelFuse(ByVal fusebits As String, ByVal bit As Byte) As String

        If Mid(fusebits, 8 - bit, 1) = "0" Then
            'si el bit es 0 se retorna un FALSE
            ObtenEstadoDelFuse = False
        Else
            'si el bit es 1 se retrona un TRUE
            ObtenEstadoDelFuse = True
        End If

    End Function


    'Funcion que configura el bit de peso i del byte de fuses recibido mediante fusebits,
    'segun el estado recibido mediante la variable estado . Recordar que para los
    'fuse bits:
    '           '0' = programmed,   TRUE
    '           '1' = unprogrammed. FALSE
    Public Sub ConfiguraEstadoDelFuse(ByRef fusebits As String, ByVal bit As Byte, ByVal estado As Boolean)

        If estado = True Then
            'si el fuse esta activo se pone a '1'
            Mid(fusebits, 8 - bit, 1) = "1"
        Else
            'si el fuse esta desactivado se pone a '0'
            Mid(fusebits, 8 - bit, 1) = "0"
        End If

    End Sub


    'Funcion que retorna un TRUE si el lock bit de peso i se encuentra activo, y retorna
    'un FALSE si se encuentra inactivo. Recoradar que para los lock bits:
    '           '0' = programmed,   TRUE
    '           '1' = unprogrammed. FALSE
    Public Function ObtenEstadoDelLock(ByVal lockbits As String, ByVal bit As Byte) As String

        If Mid(lockbits, 8 - bit, 1) = "0" Then
            'si el bit es 0 se retorna un False
            ObtenEstadoDelLock = False
        Else
            'si el bit es 1 se retrona un True
            ObtenEstadoDelLock = True
        End If

    End Function


    'Funcion que configura el bit de peso i del byte de lock bits recibido mediante lockbits
    'segun el estado recibido mediante la variable estado . Recordar que para los lock bits :
    '           '0' = programmed,   TRUE
    '           '1' = unprogrammed. FALSE
    Public Sub ConfiguraEstadoDelLock(ByRef lockbits As String, ByVal bit As Byte, ByVal estado As Boolean)

        If estado = True Then
            'se el bit a '1'
            Mid(lockbits, 8 - bit, 1) = "1"
        Else
            'si el fuse esta desactivado se pone a '0'
            Mid(lockbits, 8 - bit, 1) = "0"
        End If

    End Sub

End Module
