Module Memoria
    
    'Contiene:
    '-Classe Memoria: la classe memoria permite almacenar la informacion de cualquier tipo de memoria, independientemente
    'de su tamao, o del numero de bits de sus casillas.Por tanto contiene la informacion de la memoria, con todas la
    'variables para el control de esta. Tambien continene el atributo CadenaHex que no e mas que una parte de Contenido 
    'en una cadena de texto, asi se puede mostrar en forma de texto com p.ej una Textbox, tambien va acompaado de los
    'atributos necesarios para controlar que este texto se muestre bien. Tambien incorpora todos los metodos necesarios
    'para manipular Contenido y CadenaTextoHex, e incluso incorpora un metodo que permite actualizar contenido a partir 
    'de un archivo .hex, es decir abrir y cargar un archivo .HEX

    '-Classe MemoriaAtmel8: esta classe consta de 2 objetos de la classe Memoria. Uno de ellos es Flash con el codigo,
    'y el otro es Eprom, con los datos. Ademas incorpora los metodos necesarios para programar, leer y verificar,
    'el contenido de los objetos Flash y Eprom en el microcontrolador.

    Public Class ClasseMemoria
        'Atributos correspondientes a la informacion de la estructura
        Public BitsDato As Integer 'Indica el numero de bits de cada posicion
        Public Contenido(,) As String 'Es la tabla con el contenido del archivo .HEX
        Public Filas As Integer 'Atributo con el numero total de filas que tiene la tabla
        Public Columnas As Integer 'Atributo con el numero de columnas que tiene la tabla
        'Atributos correspondientes a la represetnacion en la cadena de texto que es la que se muestra por las textbox.
        'Estos controlan la distribucion de la informacion del archivoHEX en CadenaTextoHEX que se mostrara por alguna textbox
        Public CadenaTextoHEX As String
        Dim ColumnasTextoHEX As Integer 'Numero de columnas que se pueden mostar en la textbox donde se mostrara la cadena, asi sabemos hasta donde imprimir
        Dim ColumnasDatosTextoHEX As Integer 'Numero de columnas en que se agrupan los datos, sin contar las direcciones  
        Public FilasTextoHEX As Integer 'Numero de filas que se pueden mostar en la textbox donde se mostrara la cadena, asi sabemos hasta donde imprimir
        Public Paginas As Integer 'Numero de paginas en que se estructura la Flash
        Public TamanioPagina As Integer 'Numero de elementos que forman cada pagina

        'Procedimiento que inicializa los parametros asociados a CadenaTextoHex, la cual permite mostrar a traves de un textbox el contenido de
        'la memoria, pero antes hay que inicializar algunos parametros para que la informacion se vea bien.
        Public Sub ConfiguraCadenaTextoHEX(ByVal _ColumnasTextoHEX As Integer, ByVal _ColumnasDatosTextoHEX As Integer, ByVal _FilasTextoHEX As Integer)

            ColumnasTextoHEX = _ColumnasTextoHEX 'El "textbox" donde se mostrara la cadena esta dimensionado de forma que cabe este numero de columnas de caracteres
            ColumnasDatosTextoHEX = _ColumnasDatosTextoHEX 'En el "textbox" donde se mostrara la cadena se disponen 15 columnas de datos
            FilasTextoHEX = _FilasTextoHEX 'El "textbox" donde se mostrara la cadena esta dimensionado de forma que cabe este numero de filas

        End Sub


        'Procedimiento que rellena la CadenaTextoHEX (que se mostrara por la TextBox que toque) con el
        'Contenido. El parametro FilaOrigen indica a partir de que linea de la memoria queremos visualizar.
        'Para optimizar el proceso solo se actualiza el contenido del trozo de codigo visible, mientras
        'que la parte no visible no se carga, ya que no se ha de ver en la TextBox. Por tanto la parte 
        'del codigo que habra que mostrar es  desde la FilaOrigen hasta FilaOrigen + FilasCadenaHEX, 
        'donde FilasCadenaHEX es el ancho en filas de la TextBox donde se mostrara CadenaTextoHEX.
        Public Function RellenaCadenaTextoHEX(ByVal FilaOrigen As Integer) As String
            Dim i As Integer
            Dim j As Integer
            Dim FilaFinal As Integer
            Dim ColumnaFinal As Integer

            RellenaCadenaTextoHEX = ""

            'Se calcula cual sera la ultima Fila que se pintara en la TextBox1
            FilaFinal = FilaOrigen + FilasTextoHEX
            'En el caso de que el calculo de la FilaFinal se mayor que el numero
            'de filas del archivo, se reajusta esta valor al maximo.
            If FilaFinal > Filas - 1 Then
                FilaFinal = Filas - 1
            End If

            CadenaTextoHEX = "" 'Se inicializa a "" el contenido de la TextBox1
            For i = FilaOrigen To FilaFinal 'Se van insertando filas, con saltos de carro, en la TextBox1
                'Insercion de la cabecera de la fila: ":direccion"
                CadenaTextoHEX = CadenaTextoHEX + ":" + NatAWordHex(i * Columnas) + " "
                'Se mostraran todas las columnas de la fila de ArchivoCargado tratada (ArchivoCargado.Filas) en la
                'textbox1 y si no caben todas la columnas, se imprimiran las que quepan (ColumnaDatosTexBox1-1)
                If ((Columnas) > (ColumnasDatosTextoHEX - 1)) Then
                    ColumnaFinal = ColumnasDatosTextoHEX - 1
                Else
                    ColumnaFinal = Columnas - 1
                End If
                'Comienzan a mostrarse las columnas de la fila tratada en la TextBox1
                For j = 0 To ColumnaFinal 'Se insertan tantos datos del objeto ArchivoHex como datos quepan en cada fila
                    CadenaTextoHEX = CadenaTextoHEX + " " + Contenido(i, j)
                Next j
                'Insercion del salto de carro para marcar final de linea
                CadenaTextoHEX = CadenaTextoHEX + Chr(13) + Chr(10)
            Next i

        End Function


        'Procedimiento para la actualizacion de Contenido. Se actualiza desde la direccion
        'origen hasta la direccion anterior a la destino con "FF" o "FFFF", y en la 
        'direccion de destino se situa el dato recibido. Si la direccion origen y la 
        'destino son la misma solo se actualizara la direcion destino con el dato, es 
        'decir que no hara falta colocar ninguna "FF" o "FFFF".
        Public Sub SituaHasta(ByVal direccion_origen As Integer, ByVal direccion_destino As Integer, ByVal dato As String)
            Dim i As Integer
            Dim j As Integer
            Dim k As Integer

            'Se comprueba que la direccion recibida sea valida, si no lo es no se ejecuta nada
            If (direccion_destino < Filas * Columnas) Then
                'Las posiciones que quedan entre la direccion origen y la que 
                'hay que escribir se llenan con 'FF'
                j = Fix(direccion_origen / Columnas)  'se obtiene la fila correspondiente al origen de la escritura
                k = direccion_origen Mod Columnas 'se obtiene la columna correspondiente al origen de la escritura
                For i = direccion_origen To direccion_destino - 1
                    'Al llegar al final de la fila se salta a la siguiente
                    If (k = Columnas) Then
                        j = j + 1
                        k = 0
                    End If
                    'Se coloca un dato vacio en la posicion no ocupada
                    Select Case BitsDato
                        Case 8
                            Contenido(j, k) = "FF"
                        Case 16
                            Contenido(j, k) = "FFFF"
                        Case Else
                            Contenido(j, k) = "FF"
                    End Select
                    k = k + 1
                Next i
                'Se guarda el dato en la posicion que toca
                j = Fix(direccion_destino / Columnas) 'se obtiene la fila correspondiente al origen de la escritura
                k = direccion_destino Mod Columnas 'se obtiene la columna correspondiente al origen de la escritura
                Contenido(j, k) = dato
            End If

        End Sub


        'Funcion que lee longitud bytes de un fichero y los retorna concatenados en forma de cadena
        'Retorna un False si hay algun tipo de error durante la lectura y un Ture si todo va bien.
        Public Function LeeBytesFichero(ByVal num_fichero As Integer, ByVal longitud As Integer, ByRef cadena As String) As Boolean
            Dim i As Integer
            Dim byte_leido As Byte

            'Se inicializan las variables
            LeeBytesFichero = True
            cadena = ""
            'Se comprueba que la longitud recibida sea valida
            If longitud >= 1 Then
                'Si la longitud es valida se ponen longitud caracteres en la cadena
                i = 0
                While (i < longitud) And Not EOF(num_fichero)
                    FileGet(num_fichero, byte_leido)
                    cadena = cadena + Chr(byte_leido)
                    i = i + 1
                End While
                'Si se ha salido del bucle porque se ha sobrepasado
                'el final del archivo se devuelve un errror
                If EOF(num_fichero) Then
                    LeeBytesFichero = False
                End If
            Else
                'Si la longitud no es valida se devuelve un mensaje de error
                LeeBytesFichero = False
            End If

        End Function


        'Funcion que actualiza Contenido con la informacion descrita en un archivo .HEX, ante se tiene que haber 
        'dimensionado adecuadamente Contenido, donde se guardara la informacion del archivo, con el tamao adecuado.
        'La funcion retornara un enetero que podra tomar los siguientes valores:
        '1-Todo Ok 
        '2-Error de checksum (a pesar de esto se inetenta abrir igual)
        '3-Error de formato ":"
        '4-Error de lectura indeterminado
        Public Function AbreHex(ByVal nombre_archivo As String) As Integer
            Dim fin_lectura As Boolean 'booleano para controlar la permanencia o salida en el bucle principal
            Dim reclen As Integer 'valor del campo RECLEN del registro leido
            Dim load_offset As Integer 'valor del campo LOAD_OFFSET del registro leido
            Dim rectyp As Integer 'valor del campo RECTYP del registro leido
            Dim checksum As Integer 'valr del campo checksum
            Dim checksum_acumulado As Integer 'variable donde se va acumulando la suma de los valores leidos
            Dim cadena_leida As String 'variable auxiliar para la lectura de los archivos            
            Dim valor As Integer 'valor a almacenar en memoria
            Dim ultima_direccion_escrita As Integer 'variable que indica qual es la ultima direccion en que se ha escrito
            Dim cadena_auxiliar As String
            Dim i As Integer

            On Error GoTo gestion_error

            'Antes de abrir el archivo Contenido debe estar correctamente inicializado y 
            'dimensionado para poder asi almacenar de forma adecuada los valores cargados.

            'Se abre el fichero con el nombre_indicado en modo de acceso aleatorio para lectura, el 
            'ultimo de todos los campos indica el numero de caracteres ledios en cada acceso.
            FileOpen(1, nombre_archivo, OpenMode.Random, OpenAccess.Read, OpenShare.Default, 1)
            'Inicializacion de las variables auxiliares
            ultima_direccion_escrita = 0
            fin_lectura = False
            AbreHex = 1
            cadena_leida = ""
            While Not fin_lectura
                LeeBytesFichero(1, 1, cadena_leida)
                If cadena_leida <> ":" Then
                    fin_lectura = True
                    AbreHex = 3
                Else
                    checksum_acumulado = 0
                    'Lectura de RECLEN
                    LeeBytesFichero(1, 2, cadena_leida)
                    reclen = ByteHexANat(cadena_leida)
                    checksum_acumulado = checksum_acumulado + reclen 'Actualizacion el checksum acumulado
                    'Lectura de LOAD_OFFSET
                    LeeBytesFichero(1, 4, cadena_leida)
                    load_offset = WordHexANat(cadena_leida)
                    Select Case BitsDato
                        Case 8
                            'en el caso de que los datos de la memoria sean de 8 bits como los archivos .hex 
                            'utilizados trabajan con posiciones de 8 bits la direccion que estos dan es
                            'correcta y no hace falta dividirla por nada, es la direccio tal qual.
                            load_offset = load_offset
                        Case 16
                            'como los archivos .hex utilizados trabajan con posiciones de 8 bits, cada dos 
                            'posiciones del archivo son una posicion en la memoria, por eso la direccion
                            'leida del archivo hay que dividirla por 2
                            load_offset = Int(load_offset / 2)
                        Case Else
                    End Select
                    checksum_acumulado = checksum_acumulado + ByteHexANat(Mid(cadena_leida, 1, 2)) + ByteHexANat(Mid(cadena_leida, 3, 2)) 'Actualizacion el checksum acumulado
                    'Lectura de RECTYP
                    LeeBytesFichero(1, 2, cadena_leida)
                    rectyp = ByteHexANat(cadena_leida)
                    checksum_acumulado = checksum_acumulado + rectyp 'Actualizacion el checksum acumulado
                    'Lectura de los bytes DATA o INFO. Estos seran diferentes en funcion del tipo de registro
                    Select Case cadena_leida
                        Case "00" 'DATA REGISTER
                            'En funcion del numero de bits por data la lectura se hara de una u otra forma
                            Select Case BitsDato
                                Case 8
                                    'Lectura de cada uno los bytes de datos del registro de datos. Si los
                                    'datos son de 8 bits, se leen dos caracteres ASCII es decir 1 byte
                                    For i = 1 To reclen
                                        LeeBytesFichero(1, 2, cadena_leida)
                                        valor = ByteHexANat(cadena_leida) 'lectura del valor
                                        checksum_acumulado = checksum_acumulado + valor 'actualizacion el checksum acumulado
                                        SituaHasta(ultima_direccion_escrita, load_offset, NatAByteHex(valor))
                                        load_offset = load_offset + 1
                                        ultima_direccion_escrita = load_offset
                                    Next i
                                Case 16
                                    'Lectura de cada uno los bytes de datos del registro de datos. Si los datos 
                                    'son de 16 bits, se leen de golpe cuatro caracteres ASCII es decir 2 bytes
                                    For i = 1 To reclen Step 2
                                        LeeBytesFichero(1, 4, cadena_leida)
                                        'Como en la cadena de 4 caracteres leida del archivo primero aparece el byte de la parte baja y luego el byte 
                                        'de la parte alta (bbaa), hay que invertirlos antes de colocarlos en su posicion en Contenido
                                        cadena_auxiliar = Mid(cadena_leida, 1, 2) 'se coge el byte de la parte alta
                                        cadena_leida = Mid(cadena_leida, 3, 2) + cadena_auxiliar 'se situa primero el byte de la parte alta y luego el de la parte baja
                                        valor = ByteHexANat(Mid(cadena_leida, 1, 2)) 'lectura del valor
                                        valor = valor + ByteHexANat(Mid(cadena_leida, 3, 4)) 'lectura del valor                                
                                        checksum_acumulado = checksum_acumulado + valor 'actualizacion el checksum acumulado
                                        SituaHasta(ultima_direccion_escrita, load_offset, cadena_leida)
                                        load_offset = load_offset + 1
                                        ultima_direccion_escrita = load_offset
                                    Next i
                                Case Else
                            End Select
                        Case "01" 'END OF FILE REGISTER
                            'Este tipo de registros no contienen campo DATA o INFO
                            'por tanto no hace falta leer nada                         
                            fin_lectura = True
                        Case Else
                            '02-REGISTRO DE DIRECCION EXTENDIDA DE SEGMENTO
                            '03-REGISTRO DE DIRECCION DE INICIO DE SEGMENTO
                            '04-REGISTRO DE DIRECCION LIENAL EXTENDIDA
                            '05-REGISTRO DE INICIO DE DIRECCION LINEAL
                            'Para los demas casos e leen de todos los siguientes RECLEN bytes del registro pero no se hace nada con ellos, solo se va guardando 
                            'el (checksum_acumulado) para hacer mas tarde la comprobacion del checksum, por lo demas, los datos leidos no se usan para nada mas.
                            For i = 1 To reclen
                                LeeBytesFichero(1, 2, cadena_leida)
                                valor = ByteHexANat(cadena_leida) 'lectura del valor
                                checksum_acumulado = checksum_acumulado + valor 'actualizacion el checksum acumulado
                            Next i
                    End Select
                    'Lectura del byte de CHECKSUM
                    LeeBytesFichero(1, 2, cadena_leida)
                    checksum = ByteHexANat(cadena_leida)
                    'El byte de checksum sumado a las suma de todos los bytes del registro ha
                    'de dar = 0 si da diferente de 0 es que se ha producido algun error. 
                    If NatAByteHex(checksum + checksum_acumulado) <> "00" Then
                        AbreHex = 2
                    End If
                    LeeBytesFichero(1, 2, cadena_leida)
                End If
            End While
            'Se cierra el fichero
            FileClose(1)
            Exit Function

gestion_error:
            'Si se produce un error de lectura se retorna el valor de 
            'error de lectura y se cierra el fichero.
            AbreHex = 4
            'Se cierra el fichero
            FileClose(1)

        End Function


        Public Function PrimeraPosicionLibre() As Integer
            Dim i As Integer
            Dim j As Integer
            Dim salir As Boolean 'booleano que controla el momento en que se sale del bucle

            'Se recorre la tabla del contenido de la memoria, del final hacia el principio
            'hasta encontrar una casilla cuyo contenido sea diferente de FF, si las posicones 
            'son de 8 bits, o de FFFF si son de 16 bits.
            salir = False
            i = Filas 'como en el bucle comienzo restandole 1 no pongo el -1
            Do
                i = i - 1
                j = Columnas 'como en el bucle comienzo restandole 1 no pongo el -1
                Do
                    j = j - 1
                    'En el momento que aparece la primera direccion con el contenido diferente
                    'de "FF" o de "FFFF" se pone salir=True y se termina la busqueda.
                    Select Case BitsDato
                        Case 8
                            If Contenido(i, j) <> "FF" Then
                                salir = True
                                'como esta posicion no esta libre, la libre no es esta sino la 
                                'anterior es decir que la primera libre no es j sino j+1
                                j = j + 1
                            End If
                        Case 16
                            If Contenido(i, j) <> "FFFF" Then
                                salir = True
                                'como esta posicion no esta libre, la libre no es esta sino la 
                                'anterior es decir que la primera libre no es j sino j+1
                                j = j + 1
                            End If
                        Case Else
                            If Contenido(i, j) <> "FF" Then
                                salir = True
                                'como esta posicion no esta libre, la libre no es esta sino la 
                                'anterior es decir que la primera libre no es j sino j+1
                                j = j + 1
                            End If
                    End Select
                Loop While (j > 0) And (Not salir)
            Loop While (i > 0) And (Not salir)
            PrimeraPosicionLibre = (i * Columnas) + j

        End Function


        'Metodo que inicializa todas las casilla de la memoria a "FF" o "FFFF"
        Public Sub Inicializa()
            Dim i As Integer
            Dim j As Integer

            'Inicializacion de la tabla posicion a posicion
            For i = 0 To Filas - 1
                For j = 0 To Columnas - 1
                    'Se coloca un dato "vacio" en cada posicion
                    Select Case BitsDato
                        Case 8
                            Contenido(i, j) = "FF"
                        Case 16
                            Contenido(i, j) = "FFFF"
                        Case Else
                            Contenido(i, j) = "FF"
                    End Select
                Next j
            Next i

        End Sub


        'Metodo constructor con parametros
        Public Sub New(ByVal _Filas As Integer, ByVal _Columnas As Integer, ByVal _BitsDato As Integer)

            'Inicializacion de los atributos Filas y Columnas
            Filas = _Filas
            Columnas = _Columnas
            BitsDato = _BitsDato
            'Se le asigna el tamao adecuado a la tabla
            ReDim Contenido(Filas - 1, Columnas - 1)
            'Se inicializa la tabla posicion a posicion
            Inicializa()

        End Sub


        'Metodo destructor
        Public Sub Dispose()

            'Inicializacion de los atributos Filas y Columnas
            Filas = 0
            Columnas = 0
            BitsDato = 0
            'Se elimina todo el espacio asociado a la tabla
            ReDim Contenido(Filas, Columnas)

        End Sub

    End Class


    Public Class ClasseMemoriaAtmel8
        Public Flash As Object 'viene a ser un "puntero" universal que luego usare para apuntar a un objeto de tipo memoria
        Public Eprom As Object 'viene a ser un "puntero" universal que luego usare para apuntar a un objeto de tipo memoria
        Public Progreso As Integer  'variable que permite saber que porcentaje del proceso de programacion se ha realizado

        'Metodo constructor por defecto. Al crearse un objeto de tipo ClaseMemoriaAtmel8 sin parametros se crean la 
        'memoria Flash y la memoria Eprom del microcontrolador con las dimensiones por defecto
        Public Sub New()
            'Crea los objetos memoria con las dimensiones por defecto: dimensiones de un AVR de 8Ks de memoria de programa y 512 bytes de Eprom
            Flash = New ClasseMemoria(512, 8, 16)
            Eprom = New ClasseMemoria(64, 8, 8)
            'Los AVR de 8Ks no se estructuran en paginas, por tanto las variables que indicane el numero de paginas y su tamao permanecen a 0
            Flash.Paginas = 0
            Flash.TamanioPagina = 0
        End Sub


        'Metodo constructor con parametros. Al crearse un objeto de tipo ClaseMemoriaAtmel8 se crean la memoria Flash 
        'y la memoria Eprom del microcontrolador con las dimensiones especificadas mediante los parametros
        Public Sub New(ByVal FilasFlash As Integer, ByVal ColumnasFlash As Integer, ByVal BitsDatoFlash As Integer, ByVal FilasEprom As Integer, ByVal ColumnasEprom As Integer, ByVal BitsDatoEprom As Integer)
            Flash = New ClasseMemoria(FilasFlash, ColumnasFlash, BitsDatoFlash)
            Eprom = New ClasseMemoria(FilasEprom, ColumnasEprom, BitsDatoEprom)
        End Sub


        'Funcion que programa el contenido de Flash en la memoria Flash del
        'microcontrolador AVR. Este usa el metodo mas eficiente que consiste en
        'modificar solo las direcciones cuyo contenido sea diferente de "FFFF", el 
        'resto supone que esta a "FFFF" y no hace falta sobreescribirlas. Este metodo 
        'es aconsejable usarlo siempre que antes se haya hecho un reset de la memoria 
        'de programa. Si la programacion va bien retorna un True, mientras que si hay 
        'algun error (solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionRapidaFlashAVR8K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionRapidaFlashAVR8K = False
            Else
                primera_posicion_libre = Flash.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se programaran.
                'Si no se produce ningun error al enviar el comando de programacion se programa el contenido de Flash en el microcontrolador.
                direccion = 0 'se comienza por la direccion 00 que se
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        'Si el contenido de la direccion es "FFFF" no se escribe nada. Solo se considera el caso de "FFFF"
                        'porque esto es la prgramacion de un AVR y los AVR tienen palabras de codiogo de 16bits.
                        If Flash.Contenido(i, j) <> "FFFF" Then
                            ComandoWriteProgramMemoryBajoAVR(NatAWordBin(direccion), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), echo)
                            ComandoWriteProgramMemoryAltoAVR(NatAWordBin(direccion), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                            'En el databook se indica que despues de escribir un dato en la memoria hay que esperear a que esta haya grabado el dato anterior
                            'Existen dos opciones: una es hacer polling sobre le microcontrolador antes de escribir la siguiente direccion a la espera que nos
                            'responda que esta listo, o la otra es esperar siempre una cantidad de tiempo fija (Twd_prog mirar tabla del datasheet)
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                            ultima_posicion_programada = direccion 'se actualiza la variable ultima_posicion_prgramada por tal de calcular el progreso
                        End If
                        direccion = direccion + 1

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_programada / (primera_posicion_libre)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                    Next j
                Next i
                ProgramacionRapidaFlashAVR8K = True
            End If
            Progreso = 0 'se inicializa la variable para la proxima vez
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido de Flash en la memoria Flash del
        'microcontrolador ATMega8K. Este usa el metodo mas eficiente que consiste en
        'modificar solo las direcciones cuyo contenido sea diferente de "FFFF", el 
        'resto supone que esta a "FFFF" y no hace falta sobreescribirlas. Este metodo 
        'es aconsejable usarlo siempre que antes se haya hecho un reset de la memoria 
        'de programa. Si la programacion va bien retorna un True, mientras que si hay 
        'algun error (solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionRapidaFlashATMega8K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'variable que se va a actualizar
            Dim direccionH As Integer 'variable que se va a actualizar
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionRapidaFlashATMega8K = False
            Else
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                primera_posicion_libre = Flash.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se programaran.
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        'Si el contenido de la direccion es "FFFF" no se escribe nada. Solo se considera el caso de "FFFF"
                        'porque esto es la prgramacion de la Flash y la Flash tiene palabras de codiogo de 16bits.
                        If Flash.Contenido(i, j) <> "FFFF" Then
                            ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                            'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                            ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual
                        End If
                        direccionL = direccionL + 1 'Se incrementan los bits bajos de direccion la direccion
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega8K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_programada / (primera_posicion_libre)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                    Next j
                Next i
                ProgramacionRapidaFlashATMega8K = True
            End If
            Progreso = 0 'se inicializa la variable para la proxima vez
            SetResetNotSelect()

        End Function



        'Funcion que programa el contenido de Flash en la memoria Flash del
        'microcontrolador ATMega16K. Este usa el metodo mas eficiente que consiste en
        'modificar solo las direcciones cuyo contenido sea diferente de "FFFF", el 
        'resto supone que esta a "FFFF" y no hace falta sobreescribirlas. Este metodo 
        'es aconsejable usarlo siempre que antes se haya hecho un reset de la memoria 
        'de programa. Si la programacion va bien retorna un True, mientras que si hay 
        'algun error (solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionRapidaFlashATMega16K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'variable que se va a actualizar
            Dim direccionH As Integer 'variable que se va a actualizar
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionRapidaFlashATMega16K = False
            Else
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                primera_posicion_libre = Flash.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se programaran.
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        'Si el contenido de la direccion es "FFFF" no se escribe nada. Solo se considera el caso de "FFFF"
                        'porque esto es la prgramacion de la Flash y la Flash tiene palabras de codiogo de 16bits.
                        If Flash.Contenido(i, j) <> "FFFF" Then
                            ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                            'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                            ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual
                        End If
                        direccionL = direccionL + 1 'Se incrementan los bits bajos de direccion la direccion
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega16K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_programada / (primera_posicion_libre)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                    Next j
                Next i
                ProgramacionRapidaFlashATMega16K = True
            End If
            Progreso = 0 'se inicializa la variable para la proxima vez
            SetResetNotSelect()

        End Function



        'Funcion que programa el contenido de Flash en la memoria Flash del
        'microcontrolador ATMega32K. Este usa el metodo mas eficiente que consiste en
        'modificar solo las direcciones cuyo contenido sea diferente de "FFFF", el 
        'resto supone que esta a "FFFF" y no hace falta sobreescribirlas. Este metodo 
        'es aconsejable usarlo siempre que antes se haya hecho un reset de la memoria 
        'de programa. Si la programacion va bien retorna un True, mientras que si hay 
        'algun error (solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionRapidaFlashATMega32K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'variable que se va a actualizar
            Dim direccionH As Integer 'variable que se va a actualizar
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionRapidaFlashATMega32K = False
            Else
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                primera_posicion_libre = Flash.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se programaran.
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        'Si el contenido de la direccion es "FFFF" no se escribe nada. Solo se considera el caso de "FFFF"
                        'porque esto es la prgramacion de un AVR y los AVR tienen palabras de codiogo de 16bits.
                        If Flash.Contenido(i, j) <> "FFFF" Then
                            ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                            'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                            ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual
                        End If
                        direccionL = direccionL + 1 'Se incrementan los bits bajos de direccion la direccion
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega32K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_programada / (primera_posicion_libre)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                    Next j
                Next i
                ProgramacionRapidaFlashATMega32K = True
            End If
            Progreso = 0 'se inicializa la variable para la proxima vez
            SetResetNotSelect()

        End Function



        'Funcion que programa el contenido de Flash en la memoria Flash del
        'microcontrolador ATMega64K. Este usa el metodo mas eficiente que consiste en
        'modificar solo las direcciones cuyo contenido sea diferente de "FFFF", el 
        'resto supone que esta a "FFFF" y no hace falta sobreescribirlas. Este metodo 
        'es aconsejable usarlo siempre que antes se haya hecho un reset de la memoria 
        'de programa. Si la programacion va bien retorna un True, mientras que si hay 
        'algun error (solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionRapidaFlashATMega64K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'variable que se va a actualizar
            Dim direccionH As Integer 'variable que se va a actualizar
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionRapidaFlashATMega64K = False
            Else
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                primera_posicion_libre = Flash.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se programaran.
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        'Si el contenido de la direccion es "FFFF" no se escribe nada. Solo se considera el caso de "FFFF"
                        'porque esto es la prgramacion de un AVR y los AVR tienen palabras de codiogo de 16bits.
                        If Flash.Contenido(i, j) <> "FFFF" Then
                            ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                            'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                            ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual
                        End If
                        direccionL = direccionL + 1 'Se incrementan los 5 bits bajos de direccion la direccion
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega64K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_programada / (primera_posicion_libre)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                    Next j
                Next i
                ProgramacionRapidaFlashATMega64K = True
            End If
            Progreso = 0 'se inicializa la variable para la proxima vez
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido del objeto Flash en la memoria Flash
        'del microcontrolador AVR. Este usa el metodo menos eficiente que consiste en
        'programar todas y cada una de las casillas de la memoria con el contenido. Si
        'la programacion va bien retorna un True, mientras que si hay algun error 
        '(solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionCompletaFlashAVR8K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionCompletaFlashAVR8K = False
            Else
                ultima_posicion = Flash.Filas * Flash.Columnas - 1 'contiene la ultima direccion de la memoria Flash asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                direccion = 0 'se comienza por la direccion 00 que se
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        ComandoWriteProgramMemoryBajoAVR(NatAWordBin(direccion), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), echo)
                        ComandoWriteProgramMemoryAltoAVR(NatAWordBin(direccion), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                        'En el databook se indica que despues de escribir un dato en la memoria hay que esperear a que esta haya grabado el dato anterior
                        'Existen dos opciones: una es hacer polling sobre le microcontrolador antes de escribir la siguiente direccion a la espera que nos
                        'responda que esta listo, o la otra es esperar siempre una cantidad de tiempo fija (Twd mirar tabla del datasheet)
                        Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        direccion = direccion + 1
                    Next j

                    ultima_posicion_programada = direccion 'se podria usar directamente la variable direccion, pero prefiero hacerlo asi para hacerlo igual que en el resto de procedimientos
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_programada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()

                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                ProgramacionCompletaFlashAVR8K = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido del objeto Flash en la memoria Flash
        'del microcontrolador ATMega8K (8515, 8535). Este usa el metodo menos 
        'eficiente que consiste en programar todas y cada una de las casillas de la memoria con el contenido. Si
        'la programacion va bien retorna un True, mientras que si hay algun error 
        '(solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionCompletaFlashATMega8K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'sera la parte baja de la direccion que se va a modificar
            Dim direccionH As Integer 'sera la parte alta de la direccion que se va a modificar
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ' PROVISIONAL
            '           Flash.TamanioPagina = 
            '           Flash.Paginas = 
            ' FIN PROVISIONAL

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionCompletaFlashATMega8K = False
            Else
                ultima_posicion = Flash.Filas * Flash.Columnas - 1 'contiene la ultima direccion de la memoria Flash asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                        direccionL = direccionL + 1
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega8K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If
                    Next j

                    'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                    ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual )
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_programada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()
                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                ProgramacionCompletaFlashATMega8K = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido del objeto Flash en la memoria Flash
        'del microcontrolador ATMega16K. Este usa el metodo menos eficiente que consiste en
        'programar todas y cada una de las casillas de la memoria con el contenido. Si
        'la programacion va bien retorna un True, mientras que si hay algun error 
        '(solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionCompletaFlashATMega16K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'sera la parte baja de la direccion que se va a modificar
            Dim direccionH As Integer 'sera la parte alta de la direccion que se va a modificar
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ' PROVISIONAL
            '           Flash.TamanioPagina = 
            '           Flash.Paginas = 
            ' FIN PROVISIONAL

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionCompletaFlashATMega16K = False
            Else
                ultima_posicion = Flash.Filas * Flash.Columnas - 1 'contiene la ultima direccion de la memoria Flash asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                        direccionL = direccionL + 1
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega16K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If
                    Next j

                    'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                    ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual )
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_programada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()
                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                ProgramacionCompletaFlashATMega16K = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido del objeto Flash en la memoria Flash
        'del microcontrolador ATMega32K. Este usa el metodo menos eficiente que consiste en
        'programar todas y cada una de las casillas de la memoria con el contenido. Si
        'la programacion va bien retorna un True, mientras que si hay algun error 
        '(solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionCompletaFlashATMega32K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'sera la parte baja de la direccion que se va a modificar
            Dim direccionH As Integer 'sera la parte alta de la direccion que se va a modificar
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ' PROVISIONAL
            '          Flash.TamanioPagina = 128
            '          Flash.Paginas = 256
            ' FIN PROVISIONAL

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionCompletaFlashATMega32K = False
            Else
                ultima_posicion = Flash.Filas * Flash.Columnas - 1 'contiene la ultima direccion de la memoria Flash asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                        direccionL = direccionL + 1
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega32K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If
                    Next j

                    'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                    ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual )
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_programada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()
                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                ProgramacionCompletaFlashATMega32K = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido del objeto Flash en la memoria Flash
        'del microcontrolador ATMega64K. Este usa el metodo menos eficiente que consiste en
        'programar todas y cada una de las casillas de la memoria con el contenido. Si
        'la programacion va bien retorna un True, mientras que si hay algun error 
        '(solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionCompletaFlashATMega64K() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccionL As Integer 'sera la parte baja de la direccion que se va a modificar
            Dim direccionH As Integer 'sera la parte alta de la direccion que se va a modificar
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ' PROVISIONAL
            '          Flash.TamanioPagina = 128
            '          Flash.Paginas = 256
            ' FIN PROVISIONAL

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionCompletaFlashATMega64K = False
            Else
                ultima_posicion = Flash.Filas * Flash.Columnas - 1 'contiene la ultima direccion de la memoria Flash asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se programa el contendio de Flash en el microcontrolador
                direccionL = 0 'se comienza por la direccion baja 0
                direccionH = 0 'se comienza por la direccion alta 0
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        ComandoLoadProgramMemoryPage(NatAWordBin(direccionL), ByteHexABin(Mid(Flash.Contenido(i, j), 3, 2)), ByteHexABin(Mid(Flash.Contenido(i, j), 1, 2)), echo)
                        direccionL = direccionL + 1
                        'Controla si se ha llenado ya una pagina, si es asi se escribe en Flash y se resetea el contador
                        If direccionL = Flash.TamanioPagina Then
                            ComandoWriteProgramMemoryPageATMega64K(NatAByteBin(direccionH), echo)
                            direccionH = direccionH + 1
                            direccionL = 0
                            Pausa(Twd_Flash) 'Twd_prog=4ms de pausa segun datasheet
                        End If
                    Next j

                    'Actualizacion de la variable a partir de la cual se calcula el progreso de programacion
                    ultima_posicion_programada = (direccionH * Flash.TamanioPagina) + direccionL 'la cantidad de memoria programada es igual al ( (tamao_de_pagina * num_paginas_programadas)+direccion actual )
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_programada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()
                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                ProgramacionCompletaFlashATMega64K = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que lee todo el contenido de la memoria de programa del microcontrolador
        'y lo carga en el Contendio de Flash. Si la programacion va bien retorna un True,
        'mientras que si hay algun error (solo se controla el error en la activacion)
        ' retorna False.
        Public Function LecturaCompletaFlash() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim dato As String
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_leida As Integer 'variable donde se guarda la ultima direccion leida por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                LecturaCompletaFlash = False
            Else
                ultima_posicion = Flash.Filas * Flash.Columnas - 1 'contiene la ultima direccion de la memoria Flash asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se lee la Falsh 
                'del micrcocontrolador y se guarda en Contenido de Flash
                direccion = 0 'se comienza por la direccion 00 que se
                'Se recorre cada una de las filas
                For i = 0 To Flash.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Flash.Columnas - 1
                        dato = ComandoReadProgramMemoryBajo(NatAWordBin(direccion), echo)
                        dato = ComandoReadProgramMemoryAlto(NatAWordBin(direccion), echo) + dato
                        Flash.Contenido(i, j) = WordBinAHex(dato)
                        direccion = direccion + 1
                    Next j

                    ultima_posicion_leida = direccion 'se podria usar directamente la variable direccion, pero prefiero hacerlo asi para hacerlo igual que en el resto de procedimientos
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_leida / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()

                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                LecturaCompletaFlash = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que verifica que el contenido de Flash coincide con el contenido 
        'memoria Flash del microcontrolador AVR. Este usa el de la metodo menos 
        'eficiente que consiste en verificar todas y cada una de las posiciones 
        'de la memoria independientemente del valor de esta. Si la verificacion va 
        'bien retorna un True, mientras que si encunetra un fallo sale del bucle 
        'y retorna un False, junto a la direccion del fallo por referencia. 
        Public Function VerificacionCompletaFlash(ByRef direccion_error As Integer) As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim dato As String
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_verificada As Integer 'variable donde se guarda la ultima direccion verificada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                VerificacionCompletaFlash = False
            Else
                ultima_posicion = Flash.Filas * Flash.Columnas - 1 'contiene la ultima direccion de la memoria Flash asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se continua con la verificacion.
                direccion = 0 'se comienza por la direccion 00 que se
                direccion_error = 0
                VerificacionCompletaFlash = True
                i = 0
                j = 0
                'Se recorre cada una de las filas
                While (i <= (Flash.Filas - 1)) And (VerificacionCompletaFlash)
                    j = 0
                    'Se recorre cada una de las columnas
                    While (j <= (Flash.Columnas - 1)) And (VerificacionCompletaFlash)
                        dato = ComandoReadProgramMemoryBajo(NatAWordBin(direccion), echo)
                        dato = ComandoReadProgramMemoryAlto(NatAWordBin(direccion), echo) + dato
                        'Se comprueba si el dato leido y el contenido del archivo en esa direccion coinciden o no,
                        'de forma que si no coinciden se anota la direccion del error y se sale del bucle.
                        If Flash.Contenido(i, j) <> WordBinAHex(dato) Then
                            VerificacionCompletaFlash = False
                            direccion_error = direccion
                        End If
                        direccion = direccion + 1
                        j = j + 1
                    End While

                    ultima_posicion_verificada = direccion 'se podria usar directamente la variable direccion, pero prefiero hacerlo asi para hacerlo igual que en el resto de procedimientos
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_verificada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()

                    i = i + 1
                End While
                Progreso = 0 'se inicializa la variable para la proxima vez
            End If
            SetResetNotSelect()

        End Function


        'Funcion que verifica que el contenido de Flash coincide con el contenido
        'de la memoria de Flash del microcontrolador AVR. Este usa el metodo mas
        'eficiente que consiste en verificar solo las direcciones cuyo contenido
        'sea diferente de "FFFF", el resto supone que esta a "FFFF" y no hace falta 
        'verificarlas. Si la verificacion va bien retorna un True, mientras que 
        'si encunetra un fallo sale del bucle y retorna un False, junto a la 
        'direccion del fallo por referencia. 
        Public Function VerificacionRapidaFlash(ByRef direccion_error As Integer) As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim dato As String
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_verificada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                VerificacionRapidaFlash = False
            Else
                primera_posicion_libre = Flash.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se verificaran
                'Si no se produce ningun error al enviar el comando de programacion se programa el HEX en el microcontrolador
                direccion = 0 'se comienza por la direccion 00 que se
                direccion_error = 0
                VerificacionRapidaFlash = True
                i = 0
                j = 0
                'Se recorre cada una de las filas
                While (i <= (Flash.Filas - 1)) And (VerificacionRapidaFlash)
                    j = 0
                    'Se recorre cada una de las columnas
                    While (j <= (Flash.Columnas - 1)) And (VerificacionRapidaFlash)
                        'Si el contenido de la direccion actual es diferente de "FFFF" entonces se verifica
                        If Flash.Contenido(i, j) <> "FFFF" Then
                            dato = ComandoReadProgramMemoryBajo(NatAWordBin(direccion), echo)
                            dato = ComandoReadProgramMemoryAlto(NatAWordBin(direccion), echo) + dato
                            'Se comprueba si el dato leido y el contenido del archivo en esa direccion coinciden o no,
                            'de forma que si no coinciden se anota la direccion del error y se sale del bucle.
                            If Flash.Contenido(i, j) <> WordBinAHex(dato) Then
                                VerificacionRapidaFlash = False
                                direccion_error = direccion
                            End If
                            ultima_posicion_verificada = direccion 'se actualiza la variable ultima_posicion_verificada por tal de calcular el progreso
                        End If
                        direccion = direccion + 1

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_verificada / (primera_posicion_libre + 0.1)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                        j = j + 1
                    End While
                    i = i + 1
                End While
                Progreso = 0 'se inicializa la variable para la proxima vez
            End If
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido del objeto Eprom en la memoria Eprom
        'del microcontrolador AVR. Este usa el metodo menos eficiente que consiste en
        'programar todas y cada una de las casillas de la memoria con el contenido. Si
        'la programacion va bien retorna un True, mientras que si hay algun error 
        '(solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionCompletaEprom() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.


            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionCompletaEprom = False
            Else
                ultima_posicion = Eprom.Filas * Eprom.Columnas - 1 'contiene la ultima direccion de la memoria Eprom asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se programa el Contenido de Eprom en el microcontrolador
                direccion = 0 'se comienza por la direccion 00 que se
                'Se recorre cada una de las filas
                For i = 0 To Eprom.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Eprom.Columnas - 1
                        ComandoWriteEpromMemory(NatAWordBin(direccion), ByteHexABin(Eprom.Contenido(i, j)), echo)
                        'En el databook se indica que despues de escribir un dato en la memoria hay que esperear a que esta haya grabado el dato anterior
                        'Existen dos opciones: una es hacer polling sobre le microcontrolador antes de escribir la siguiente direccion a la espera que nos
                        'responda que esta listo, o la otra es esperar siempre una cantidad de tiempo fija (Twd mirar tabla del datasheet)
                        Pausa(Twd_Eprom) 'Twd_Eprom=9.5ms de pausa segun datasheet
                        direccion = direccion + 1
                    Next j

                    ultima_posicion_programada = direccion 'se podria usar directamente la variable direccion, pero prefiero hacerlo asi para hacerlo igual que en el resto de procedimientos
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_programada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()

                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                ProgramacionCompletaEprom = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que lee todo el contenido de la memoria Eprom del microcontrolador
        'y lo carga en la estructura de memoria. Si el proceso va bien retorna
        'un True, mientras que si hay algun error (solo se controla el error en la 
        'activacion) retorna False.
        Public Function LecturaCompletaEprom() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim dato As String
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_leida As Integer 'variable donde se guarda la ultima direccion leida por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                LecturaCompletaEprom = False
            Else
                ultima_posicion = Eprom.Filas * Eprom.Columnas - 1 'contiene la ultima direccion de la memoria Eprom asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se programa el HEX en el microcontrolador
                direccion = 0 'se comienza por la direccion 00 que se
                'Se recorre cada una de las filas
                For i = 0 To Eprom.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Eprom.Columnas - 1
                        dato = ComandoReadEpromMemory(NatAWordBin(direccion), echo)
                        Eprom.Contenido(i, j) = ByteBinAHex(dato)
                        direccion = direccion + 1

                    Next j

                    ultima_posicion_leida = direccion 'se podria usar directamente la variable direccion, pero prefiero hacerlo asi para hacerlo igual que en el resto de procedimientos
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_leida / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()

                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                LecturaCompletaEprom = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que programa el contenido de Eprom en la memoria de Eprom
        'del microcontrolador AVR. Este usa el metodo mas eficiente que consiste en
        'modificar solo las direcciones cuyo contenido sea diferente de "FF", el 
        'resto supone que esta a "FF" y no hace falta sobreescribirlas. Este metodo 
        'es aconsejable usarlo siempre que antes se haya hecho un reset de la memoria 
        'de programa. Si la programacion va bien retorna un True, mientras que si hay 
        'algun error (solo se controla el error en la activacion) retorna False.
        Public Function ProgramacionRapidaEprom() As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_programada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                ProgramacionRapidaEprom = False
            Else
                primera_posicion_libre = Eprom.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se programaran.                
                'Si no se produce ningun error al enviar el comando de programacion se programa Contenido de Eprom en el microcontrolador
                direccion = 0 'se comienza por la direccion 00 que se
                'Se recorre cada una de las filas
                For i = 0 To Eprom.Filas - 1
                    'Se recorre cada una de las columnas
                    For j = 0 To Eprom.Columnas - 1
                        'Si el contenido de la direccion es "FFFF" no se escribe nada. Solo se considera el caso de "FFFF"
                        'porque esto es la prgramacion de un AVR y los AVR tienen palabras de codiogo de 16bits.
                        If Eprom.Contenido(i, j) <> "FF" Then
                            ComandoWriteEpromMemory(NatAWordBin(direccion), ByteHexABin(Eprom.Contenido(i, j)), echo)
                            'En el databook se indica que despues de escribir un dato en la memoria hay que esperear a que esta haya grabado el dato anterior
                            'Existen dos opciones: una es hacer polling sobre le microcontrolador antes de escribir la siguiente direccion a la espera que nos
                            'responda que esta listo, o la otra es esperar siempre una cantidad de tiempo fija (Twd_prog mirar tabla del datasheet)
                            Pausa(Twd_Eprom) 'Twd_Eprom=9.5ms de pausa segun datasheet
                            ultima_posicion_programada = direccion 'se actualiza la variable ultima_posicion_prgramada por tal de calcular el progreso
                        End If
                        direccion = direccion + 1

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_programada / (primera_posicion_libre + 0.1)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                    Next j
                Next i
                Progreso = 0 'se inicializa la variable para la proxima vez
                ProgramacionRapidaEprom = True
            End If
            SetResetNotSelect()

        End Function


        'Funcion que verifica que el contenido de Erpom coincide con el contenido 
        'de la Eprom del microcontrolador AVR. Este usa el metodo menos eficiente
        'que consiste en verificar todas y cada una de las posiciones de la memoria 
        'independientemente del valor de esta. Si la verificacion va bien retorna
        'un True, mientras que si encunetra un fallo sale del bucle y retorna un 
        'False, junto a la direccion del fallo la cual la pasa por referencia. 
        Public Function VerificacionCompletaEprom(ByRef direccion_error As Integer) As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim dato As String
            Dim ultima_posicion As Integer 'variable donde se guarda el total de direcciones de la memoria
            Dim ultima_posicion_verificada As Integer 'variable donde se guarda la ultima direccion verificada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                VerificacionCompletaEprom = False
            Else
                ultima_posicion = Eprom.Filas * Eprom.Columnas - 1 'contiene la ultima direccion de la memoria Eprom asi sabremos el progreso
                'Si no se produce ningun error al enviar el comando de programacion se verifica la Eprom del microcontrolador
                direccion = 0 'se comienza por la direccion 00 que se
                direccion_error = 0
                VerificacionCompletaEprom = True
                i = 0
                j = 0
                'Se recorre cada una de las filas
                While (i <= (Eprom.Filas - 1)) And (VerificacionCompletaEprom)
                    j = 0
                    'Se recorre cada una de las columnas
                    While (j <= (Eprom.Columnas - 1)) And (VerificacionCompletaEprom)
                        dato = ComandoReadEpromMemory(NatAWordBin(direccion), echo)
                        'Se comprueba si el dato leido y el contenido del archivo en esa direccion coinciden o no,
                        'de forma que si no coinciden se anota la direccion del error y se sale del bucle.
                        If Eprom.Contenido(i, j) <> ByteBinAHex(dato) Then
                            VerificacionCompletaEprom = False
                            direccion_error = direccion
                        End If
                        direccion = direccion + 1
                        j = j + 1
                    End While

                    ultima_posicion_verificada = direccion 'se podria usar directamente la variable direccion, pero prefiero hacerlo asi para hacerlo igual que en el resto de procedimientos
                    'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                    Progreso = Int((ultima_posicion_verificada / (ultima_posicion + 0.1)) * 100)
                    'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                    System.Windows.Forms.Application.DoEvents()

                    i = i + 1
                End While
                Progreso = 0 'se inicializa la variable para la proxima vez
            End If
            SetResetNotSelect()

        End Function


        'Funcion que verifica que el contenido de Eprom coincide con el contenido
        'de la memoria Eprom del microcontrolador AVR. Este usa el metodo mas
        'eficiente que consiste en verificar solo las direcciones cuyo contenido
        'sea diferente de "FF", el resto supone que esta a "FF" y no hace falta 
        'verificarlas. Si la verificacion va bien retorna un True, mientras que 
        'si encunetra un fallo sale del bucle y retorna un False, junto a la 
        'direccion del fallo por referencia. 
        Public Function VerificacionRapidaEprom(ByRef direccion_error As Integer) As Boolean
            Dim echo As String = "00000000"
            Dim i As Integer
            Dim j As Integer
            Dim direccion As Integer 'variable que se va a actualizar
            Dim dato As String
            Dim primera_posicion_libre As Integer 'variable donde se guarda la direccion de la posicion siguiente a la ultima direccion ocupada.
            Dim ultima_posicion_verificada As Integer 'variable donde se guarda la ultima direccion programada por tal de poder calcular el progreso.

            ActivaProgramacionAVR()
            If Not ComandoProgrammingEnable(echo) Then
                'Si se produce un error al enviar el comando de programacion se retorna False
                VerificacionRapidaEprom = False
            Else
                primera_posicion_libre = Eprom.PrimeraPosicionLibre() 'indica de forma aproximada el numero de direcciones que se verificaran
                'Si no se produce ningun error al enviar el comando de programacion se verifica la Eprom
                direccion = 0 'se comienza por la direccion 00 que se
                direccion_error = 0
                VerificacionRapidaEprom = True
                i = 0
                j = 0
                'Se recorre cada una de las filas
                While (i <= (Eprom.Filas - 1)) And (VerificacionRapidaEprom)
                    j = 0
                    'Se recorre cada una de las columnas
                    While (j <= (Eprom.Columnas - 1)) And (VerificacionRapidaEprom)
                        'Si el contenido de la direccion actual es diferente de "FFFF" entonces se verifica
                        If Eprom.Contenido(i, j) <> "FF" Then
                            dato = ComandoReadEpromMemory(NatAWordBin(direccion), echo)
                            'Se comprueba si el dato leido y el contenido del archivo en esa direccion coinciden o no,
                            'de forma que si no coinciden se anota la direccion del error y se sale del bucle.
                            If Eprom.Contenido(i, j) <> ByteBinAHex(dato) Then
                                VerificacionRapidaEprom = False
                                direccion_error = direccion
                            End If
                            ultima_posicion_verificada = direccion 'se actualiza la variable ultima_posicion_verificada por tal de calcular el progreso
                            'En el databook se indica que despues de leer un dato en la memoria hay que esperear a que esta haya leido el dato anterior
                            'Existen dos opciones: una es hacer polling sobre el microcontrolador antes de leer la siguiente direccion a la espera que nos
                            'responda que esta listo, o la otra es esperar siempre una cantidad de tiempo fija (Twd mirar tabla del datasheet)
                            Pausa(Trd_Eprom)
                        End If
                        direccion = direccion + 1

                        'Actualizacion la variable progreso con el porcentaje de memoria que se lleva programada.                                                                        
                        Progreso = Int((ultima_posicion_verificada / (primera_posicion_libre + 0.1)) * 100)
                        'Tras la actualizacion de la variable progreso se hace un DoEvents para que se puedan ejecutar el resto de eventos de la aplicacion y puedan mostrar el nuevo valor de la variable, si es necesario.
                        System.Windows.Forms.Application.DoEvents()

                        j = j + 1
                    End While
                    i = i + 1
                End While
                Progreso = 0 'se inicializa la variable para la proxima vez
            End If
            SetResetNotSelect()

        End Function


        'Metodo destructor
        Public Sub Dispose()

            'Se elimina la Flah
            Flash.Dispose()
            'Se elimina la Eprom
            Eprom.Dispose()

        End Sub


        Protected Overrides Sub Finalize()
            MyBase.Finalize()
        End Sub

    End Class



End Module
